import {Component, EventEmitter, Input, Output, ViewChild} from '@angular/core'
import {
  Mix,
  MixMaterial,
  MixMaterialDto,
  mixMaterialsAsMixMaterialsDto,
  Recipe, RecipeStep,
  sortMixMaterialsDto
} from '../../../shared/model/recipe.model'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {MixService} from '../../services/mix.service'
import {RecipeService} from '../../services/recipe.service'
import {Material} from '../../../shared/model/material.model'
import {MaterialService} from '../../../material/service/material.service'
import {MaterialTypeService} from '../../../material-type/service/material-type.service'
import {FormBuilder, FormControl, FormGroup, Validators} from '@angular/forms'
import {UNIT_MILLILITER} from '../../../shared/units'
import {MatTable} from '@angular/material/table'
import {ActivatedRoute, Router} from '@angular/router'
import {ConfirmBoxComponent} from '../../../shared/components/confirm-box/confirm-box.component'
import {AccountService} from '../../../accounts/services/account.service'
import {EmployeePermission} from '../../../shared/model/employee'
import {ErrorService} from '../../../shared/service/error.service'
import {MatSelect} from '@angular/material/select'

@Component({
  selector: 'cre-mix-editor',
  templateUrl: './mix-editor.component.html',
  styleUrls: ['./mix-editor.component.sass']
})
export class MixEditorComponent extends ErrorHandlingComponent {
  @ViewChild('matTable') mixTable: MatTable<MixMaterial>
  @ViewChild('deleteConfirmBox') deleteConfirmBox: ConfirmBoxComponent

  @Input() mixId: number | null
  @Input() recipeId: number | null
  @Input() materials: Material[]

  @Output() save = new EventEmitter<any>()

  mix: Mix | null
  recipe: Recipe | null
  materialTypes$ = this.materialTypeService.all

  form: FormGroup
  nameControl: FormControl
  materialTypeControl: FormControl

  mixMaterials: MixMaterialDto[] = []
  editionMode = false
  units = UNIT_MILLILITER
  hoveredMixMaterial: MixMaterial | null
  columns = ['position', 'buttonsPosition', 'material', 'quantity', 'units', 'buttonRemove']

  deleting = false
  handledErrorModels = [{
    filter: error => error.status === 409 && !this.deleting,
    messageProducer: error => `Un mélange avec le nom '${error.id}' existe déjà dans cette recette`
  }, {
    filter: error => error.error && error.error.status === 409 && this.deleting,
    consumer: () => this.deleting = false,
    messageProducer: () => 'Ce mélange est utilisé par un ou plusieurs autres mélanges'
  }]

  constructor(
    private mixService: MixService,
    private recipeService: RecipeService,
    private materialService: MaterialService,
    private materialTypeService: MaterialTypeService,
    private accountService: AccountService,
    private formBuilder: FormBuilder,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.mixId = this.urlUtils.parseIntUrlParam('id')
    if (this.mixId) {
      this.editionMode = true
    }

    this.subscribeEntityById(
      this.recipeService,
      this.recipeId,
      r => {
        this.recipe = r
        if (this.editionMode) {
          this.mix = this.recipe.mixes.find(m => m.id === this.mixId)
          this.mixMaterials = mixMaterialsAsMixMaterialsDto(this.mix)
        } else {
          this.addBlankMixMaterial()
        }
        this.generateForm()
      },
      '/color/list'
    )
  }

  addRow() {
    this.addBlankMixMaterial()
    this.mixTable.renderRows()
  }

  removeRow(position: number) {
    this.mixMaterials.splice(position, 1)

    // Decreases the position of each mix material above the removed one
    for (let i = position; i < this.mixMaterials.length; i++) {
      this.mixMaterials[i].position -= 1
    }

    this.mixTable.renderRows()
  }

  increasePosition(mixMaterial: MixMaterialDto, table: MatTable<any>) {
    this.updateMixMaterialPosition(mixMaterial, mixMaterial.position + 1)
    this.sort(table)
  }

  decreasePosition(mixMaterial: MixMaterialDto, table: MatTable<any>) {
    this.updateMixMaterialPosition(mixMaterial, mixMaterial.position - 1)
    this.sort(table)
  }

  sort(table: MatTable<any>) {
    this.mixMaterials = sortMixMaterialsDto(this.mixMaterials)
    table.renderRows()
  }

  setMixMaterialMaterial(mixMaterial: MixMaterialDto, materialId: number) {
    mixMaterial.isPercents = this.materials.find(m => m.id === materialId).materialType.usePercentages
    mixMaterial.materialId = materialId
  }

  submit() {
    this.save.emit({
      name: this.nameControl.value,
      recipeId: this.recipeId,
      materialTypeId: this.materialTypeControl.value,
      mixMaterials: this.mixMaterials,
      units: this.units
    })
  }

  delete() {
    this.deleting = true
    this.subscribeAndNavigate(this.mixService.delete(this.mixId), `/color/edit/${this.recipeId}`)
  }

  getAvailableMaterials(mixMaterial: MixMaterialDto): Material[] {
    return this.materials.filter(m => mixMaterial.materialId === m.id || this.mixMaterials.filter(mm => mm.materialId === m.id).length === 0)
  }

  get canDeleteMix() {
    return this.accountService.hasPermission(EmployeePermission.REMOVE_RECIPE)
  }

  private generateForm() {
    this.nameControl = new FormControl(this.mix ? this.mix.mixType.name : null, Validators.required)
    this.materialTypeControl = new FormControl(this.mix ? this.mix.mixType.material.materialType.id : null, Validators.required)
    this.form = this.formBuilder.group({
      name: this.nameControl,
      materialType: this.materialTypeControl
    })
  }

  private addBlankMixMaterial() {
    this.mixMaterials.push(
      new MixMaterialDto(null, 0, false, this.mixMaterials.length + 1)
    )
  }

  private updateMixMaterialPosition(mixMaterial: MixMaterialDto, updatedPosition: number) {
    if (!this.mixMaterialAtPosition(updatedPosition)) {
      mixMaterial.position = updatedPosition
    } else {
      const conflictingStep = this.mixMaterialAtPosition(updatedPosition)
      conflictingStep.position = mixMaterial.position
      mixMaterial.position = updatedPosition
    }
  }

  private mixMaterialAtPosition(position: number): MixMaterialDto {
    return this.mixMaterials.find(m => m.position === position)
  }
}
